
=head1 NAME

POWER::Epoll - Perl extension for epoll(4)

=head1 SYNOPSIS

 use POWER::Epoll;

 my $epfd = epoll_create(10) or die $!;
 epoll_add($epfd, fileno(SOMEFD1), EPOLLET|EPOLLIN);
 epoll_add($epfd, fileno(SOMEFD2), EPOLLET|EPOLLIN|EPOLLOUT);
 epoll_mod($epfd, fileno(SOMEFD2), EPOLLET|EPOLLOUT);
 epoll_del($epfd, fileno(SOMEFD2));
 ($status, $events) = epoll_wait($epfd, $timeout);

=head1 DESCRIPTION

Interface for epoll(4) in linux kernels 2.6+. Use syscall(), so glibc
compiled with linux headers 2.6+ isn't required.

=head1 EXPORT

Constants (see epoll_ctl(2) for descriptions) :

EPOLLIN EPOLLOUT EPOLLPRI EPOLLERR EPOLLHUP EPOLLET EPOLLONESHOT

Functions:

epoll_create epoll_add epoll_mod epoll_del epoll_wait

=head1 FUNCTIONS

=over 4

=cut

package POWER::Epoll;

use Carp;
use Fcntl;
use Errno;
use strict;

our $VERSION = '1.10';

require XSLoader;
XSLoader::load('POWER::Epoll', $VERSION);

sub import {
    no strict;
    *{"${\ scalar caller}::$_"} = \&{$_} for qw(
	EPOLLIN EPOLLOUT EPOLLPRI EPOLLERR EPOLLHUP EPOLLET EPOLLONESHOT
	epoll_create epoll_add epoll_mod epoll_del epoll_wait
	);
}

=item B<epoll_create>( $size )

Execute epoll_create(2).

Return undef on error, $epfd on success.

=item B<epoll_add>( $epfd, $fileno, $events )

Execute epoll_ctl(2)  op=EPOLL_CTL_ADD.

Return undef on error, 1 on success.

=item B<epoll_mod>( $epfd, $fileno, $events )

Execute epoll_ctl(2)  op=EPOLL_CTL_MOD.

Return undef on error, 1 on success.

=item B<epoll_del>( $epfd, $fileno )

Execute epoll_ctl(2)  op=EPOLL_CTL_DEL.

Return undef on error, 1 on success.

=item B<epoll_wait>( $epfd, $timeout )

Execute epoll_wait(2). $timeout should be in seconds, not in milliseconds,
but can be fractional. Set $timeout to undef for unlimited timeout.

For now $timeout == -0.001 also mean unlimited timeout but you shouldn't
rely on this because this behaviour is side effect of converting from
seconds to milliseconds.

Return list: ($wait_status, [[$fileno1, $events1], [$fileno2, $events2], ...]).

=back

=head1 HISTORY

 1.10       Wed Nov 02 01:41:44 EET 2005
    - removed dependency on POWER::lib
    - hardcoded constants for syscall() because
      linux-headers-2.4.x doesn't contain them yet

 1.00		Fri Apr 22 19:10:44 EEST 2005
	- INCOMPATIBLE CHANGES: complete interface rewrite:
	    now this module is just wrapper around kernel functions
	    with values transformation to Perlish way
	    (undef on errors instead of -1, etc.)

 0.01		Sun Feb 22 00:06:14 EET 2004
	- initial release

=head1 AUTHOR

 Alex Efros <powerman@asdfGroup.com>
 Nikita Savin <nikita@asdfGroup.com>
 Ilya Persky <ilya_persky@yahoo.com>

=cut

1;

