#!/usr/bin/perl -w

=head1 NAME

B<POWER::Tree> -         
  (a-la    Windows Explorer).

=head1 SYNOPSIS

 $tree = POWER::Tree->root();
 $tree->delimiter("/");
 $tree->add_value("/", "point_1") or print $tree->err,"\n";
 $tree->add_value("/point_1", "point_1.1");
 $tree->add_value("/point_1/point_1.1", "point_1.1.1");
 $tree->add_value("/", "point_2");
 $tree->add_value("/point_2", "point_2.1");
 @list = @{ $tree->tree2list() };
 print map { join(" ", @$_)."\n" } @list;

 (    ):

 =-  /
     - point_1 			/point_1
     | v- point_1.1 		/point_1/point_1.1
     |    vo point_1.1.1 	/point_1/point_1.1/point_1.1.1
    v- point_2 			/point_2
       vo point_2.1 		/point_2/point_2.1
       
 $point = $tree->del_node("/point_1/point_1.1");
 $tree->add_node("/", $point);
 @list = @{ $tree->tree2list() };
 print map { join(" ", @$_)."\n" } @list;

 (    ):

 =-  /
     o point_1 			/point_1
     - point_2 			/point_2
     | vo point_2.1 		/point_2/point_2.1
    v- point_1.1 		/point_1.1
       vo point_1.1.1 		/point_1.1/point_1.1.1

 $tree->move_node("/point_1.1", -1);
 @list = @{ $tree->tree2list() };
 print map { join(" ", @$_)."\n" } @list;

 (    ):

 =-  /
     o point_1 			/point_1
     - point_1.1 		/point_1.1
     | vo point_1.1.1 		/point_1.1/point_1.1.1
    v- point_2 			/point_2
       vo point_2.1 		/point_2/point_2.1

=head1 DESCRIPTION

   ,     
  -    "/", " ", 
 "->"    .
      "/";
      "/value";
 "/" -  ,  "value" -  .

=head1 IDEA

     " ".   
  .      ,    
 ( 0)   ,    -  
   .     - :

 $ROOT = [ $root_value, $node_1, $node_2, ..., $node_N ];
 
 $node_1, $node_2  ..  - :

 $node1 = [ $node_1_value, $node_1_1, $node_1_2, ..., $node_1_N ];

  ($xxxx_value)   ,    
       .
  POWER::Tree        
   .     
   .  POWER::Tree::AnyValue.

=head1 METHODS

 ,   undef    
 ,      err().

:    ,   
 ,    ,  
        .

     POWER::Tree::AnyValue.

=over 4

=cut

package POWER::Tree;
use strict;
use vars qw($VERSION);
$VERSION = "1.01" ;

my %Tree = (
	delimiter => "/",
    );

=item B<delimiter>( $new_delimiter )

      $new_delimiter.
        - 
        .
    (  - "/").

=cut

sub delimiter {
    my $self  = shift ;
    return(@_ ? $Tree{delimiter}=shift : $Tree{delimiter}) unless ref($self) ;
    $self->{delimiter} = shift if @_ ;
    return defined($self->{delimiter})	? $self->{delimiter}
					: $Tree{delimiter} ;
}

=item B<err>()

       undef.
      ,   .

=cut

sub err {
    my $self	= shift ;
    my $err	= $self->{err} ;
    $self->{err}= undef ;
    return $err ;
}

=item B<path>( $value )

     .

=cut

sub path {
    my ($self, $value) = @_;
    return $self->delimiter . $value;
}

=item B<root>()

   (/).
        .

=cut

sub root {
    my $this  = shift ;
    my $class = ref($this) || $this ;
    my $self  = ref($this) ? { %$this } : { ROOT => [ undef ] };
    bless $self, $class ;
}

=item B<add_value>( $path, $value, $pos )

  $value  $path.
  add_node,       .

=cut

sub add_value {
    my ($self, $path, $value, $pos) = @_ ;
    $self->add_node($path, [$value], $pos);
}

=item B<add_node>( $path, $node, $pos )

  $node  $path.
  $pos -    $pos,    .
   1 .
      undef  .

=cut

sub add_node {
    my ($self, $path, $node, $pos) = @_ ;
    $path = $self->_normalize_path($path) ;
    my $current = $self->path($node->[0]) ;
    $self->{err}="bad node path ``$current''", return 
	if $current=~/^${\$self->delimiter}.*${\$self->delimiter}/;
    my $add_path = $self->_normalize_path($path . $current) ;
    $self->{err} = "``$add_path'' already exists", return
	if $self->find_node($add_path);
    $self->{err} = undef;
    return unless my $parent_node = $self->find_node($path) ;
    $pos = ! defined($pos) ? @$parent_node : $pos < 1 ? 1 : $pos ;
    splice @$parent_node, $pos, 0, $node ;
    return $add_path ;
}

=item B<del_node>( $path )

  $path.
    undef  .

=cut

sub del_node {
    my ($self, $path)  = @_ ;
    $path = $self->_normalize_path($path) ;
    return unless my ($node, $pos) = $self->find_node_pos($path) ;
    splice @$node, $pos, 1 ;
}

=item B<move_node>( $path, $move )

  $path  $move .
$move > 0 -  , $move < 0 -   (  ).
 undef  .

=cut

sub move_node {
    my ($self, $path, $move) = @_ ;
    $path = $self->_normalize_path($path) ;
    return unless my $pos = ($self->find_node_pos($path))[1];
    my ($current, @path) = reverse split $self->delimiter, $path ;
    my $parent = join($self->delimiter, reverse @path);
    $self->add_node($parent, $self->del_node($path), $pos+$move);
}

=item B<value>( $path, $new_value )

/  .
  $new_value   .
    undef  .

=cut

sub value {
    my ($self, $path) = (shift, shift);
    $path = $self->_normalize_path($path);
    my $node = $self->find_node($path);
    $self->{err}="can't set value to null", return if @_ and $_[0] eq "";
    $self->{err}="can't set root value", return if @_ and $path eq $self->delimiter;
    $node->[0] = shift if @_;
    return $node->[0];
}

=item B<find_node_pos>( $path )

    $path.
          undef 
 .

=cut

sub find_node_pos {
    my ($self, $path) 	= @_;
    $path = $self->_normalize_path($path) ;
    my ($current, @path) = reverse split $self->delimiter, $path ;
    $self->{err}="``$path'' has no parent", return unless defined $current ;
    return unless 
	my $node = $self->find_node(join $self->delimiter, reverse @path);
    $current = $self->_normalize_path($current);
    $self->{err}="``$path'' position not found", return unless
	my ($pos) = grep { $current eq $self->path($node->[$_][0]) } 1..$#$node;
    return ($node, $pos) ;
}

=item B<find_node>( $path )

  $path.
    undef  .

=cut

sub find_node {
    my $self = shift ;
    my $path = $self->_normalize_path(shift) ;
    (undef, my @path) = split $self->delimiter, $path;
    my $node = $self->{ROOT};
    for my $current (map {$self->_normalize_path($_)} @path) {
	($node) = grep { $current eq $self->path($_->[0]) } @$node[1..$#$node];
    }
    $self->{err}="``$path'' not found" unless $node;
    return $node;
}

=item B<tree2list>( \&is_open, $deep, $start_path )

    .
   .

 &is_open       .  &is_open
 ,     . 
  &is_open = sub { 1 }; ..   .

$deep  ,      . 
,      $deep=1.   $deep=0.

$start_path  ,       .

      undef  .
     ţ .
   :     ,
         .

  :

=over 4

=item B<' |'>

       

=item B<'  '>

      

=item B<' o'>

 

=item B<' +'>

 

=item B<' -'>

 

=item B<'vo'>

  ,  

=item B<'v+'>

  ,  

=item B<'v-'>

  ,  

=item B<'~o'>

   ,  

=item B<'~+'>

   ,  

=item B<'~-'>

   ,  

=item B<'=o'>

   ,  

=item B<'=+'>

   ,  

=item B<'=-'>

   ,  

=back

=cut

sub tree2list {
    my ($self, $is_open, $deep, $start_path) = @_ ;
    $is_open 	||= sub { 1 } ;
    $start_path ||= $self->path($self->{ROOT}[0]) ;
    $start_path = $self->_normalize_path($start_path);
    return unless my $start_node=$self->find_node($start_path);
    my @list;
    my @nodes = (["=", $start_node, $start_path]);
    while (my $line = shift @nodes) {
	my ($path, $node, $pos) = (pop @$line, pop @$line, pop @$line);
	my $sub = ! $#$node ? "o" : &$is_open($path) ? "-" : "+" ;
	push @list, [@$line[$deep..$#$line], $pos.$sub, $self->value($path), 
	    $path] if $deep<=@$line;
	next unless $sub eq "-";
	push @$line, ($pos eq "=" || $pos eq "v" ? "  " : " |");
	for (reverse 1..$#$node) {
	    $pos = $_==$#$node ? "v" : " ";
	    $pos = $#$node>1   ? "~" : "=" if $deep==@$line && $_==1;
	    unshift @nodes, [@$line, $pos, $node->[$_], $self->_normalize_path(
		$path. $self->delimiter. $self->path($node->[$_][0]))];
	}
    }
    return \@list;
}

=item B<_normalize_path>( $path )

 .
   ,   
       .
  .

=cut

sub _normalize_path {
    my ($self, $path) = @_;
    my $delimiter = $self->delimiter;
    $path =~ s/$delimiter*$// ;
    $path =~ s/^$delimiter*/$delimiter/ ;
    $path =~ s/$delimiter+/$delimiter/g ;
    return $path;
}

=back

=head1 AUTHOR

 Alex Efros <powerman@consultant.com>

=cut
